<?php

namespace Drupal\something_went_wrong\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\Client;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Class SomethingWentWrongExceptionSubscriber.
 *
 * @package Drupal\something_went_wrong\EventSubscriber
 */
class SomethingWentWrongExceptionSubscriber implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * SomethingWentWrongExceptionSubscriber constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory->get('something_went_wrong.settings');
  }

  /**
   * Exceptions handler.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseForExceptionEvent $event
   *   The event to process.
   */
  public function onException(GetResponseForExceptionEvent $event) {
    $exception = $event->getException();
    $class_name = get_class($exception);
    $ignore_list = explode(',', $this->configFactory->get('ignore_list'));

    $send = TRUE;

    foreach ($ignore_list as $item) {
      if ($item && strpos($class_name, $item) !== FALSE) {
        $send = FALSE;
        break;
      }
    }

    if ($send) {
      $message = $this->buildMessage($exception);
      $this->sendMessage($message);
    }
  }

  /**
   * Builds message.
   *
   * @param \Exception $exception
   *
   * @return string
   */
  protected function buildMessage(\Exception $exception) {
    $class_name = get_class($exception);
    $exception_message = $exception->getMessage();
    $exception_file = $exception->getFile();
    $exception_line = $exception->getLine();

    return sprintf("*Uncaught PHP Exception* \n Class name: %s \n Message: %s \n File: %s \n Line: %s", $class_name, $exception_message, $exception_file, $exception_line);
  }

  /**
   * Sends message to Slack.
   *
   * @param $message
   */
  protected function sendMessage($message) {
    $label = $this->configFactory->get('label');
    $webhook_url = $this->configFactory->get('webhook_url');

    if ($webhook_url) {
      $client = new Client();

      try {
        $client->post($webhook_url, [
          'body' => json_encode([
            'text' => $message,
            'username' =>  $label ?: 'Something Went Wrong Bot',
          ]),
        ]);
      }
      catch (\Exception $e) {

      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::EXCEPTION][] = ['onException', 60];
    return $events;
  }
}
