<?php

namespace Drupal\something_went_wrong\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class SettingsForm.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'something_went_wrong.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'something_went_wrong_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('something_went_wrong.settings');

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#description' => $this->t('Enter your site name or similar site identifier. This is used as a Slack username.'),
      '#maxlength' => 64,
      '#size' => 64,
      '#default_value' => $config->get('label'),
    ];

    $form['webhook_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Webhook URL'),
      '#description' => $this->t('Go to your <a target="_blank" href="https://slack.com/services/new/incoming-webhook">Slack settings</a> and add a Webhook integration. After you create the integration you will get a url, which you need to enter in this field.'),
      '#maxlength' => 256,
      '#size' => 64,
      '#default_value' => $config->get('webhook_url'),
      '#required' => TRUE,
    ];

    $form['ignore_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Ignore list'),
      '#description' => $this->t('Comma separated list of exception classes to ignore. For example: NotFoundHttpException'),
      '#default_value' => $config->get('ignore_list'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config('something_went_wrong.settings')
      ->set('label', $form_state->getValue('label'))
      ->set('webhook_url', $form_state->getValue('webhook_url'))
      ->set('ignore_list', $form_state->getValue('ignore_list'))
      ->save();
  }

}
